@@ -22,20 +22,19 @@
  */
 
 #include "config.h"
-
 #include "ephy-location-entry.h"
-#include "ephy-signal-accumulator.h"
-#include "ephy-dnd.h"
+
+#include "ephy-about-handler.h"
 #include "ephy-debug.h"
+#include "ephy-dnd.h"
 #include "ephy-gui.h"
-#include "ephy-request-about.h"
+#include "ephy-signal-accumulator.h"
 
-#include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <webkit/webkit.h>
-
 #include <string.h>
+#include <webkit/webkit.h>
 
 /**
  * SECTION:ephy-location-entry
@@ -78,6 +77,7 @@ struct _EphyLocationEntryPrivate
 	guint apply_colors : 1;
 	guint needs_reset : 1;
 	guint show_lock : 1;
+	guint show_favicon : 1;
 };
 
 static const GtkTargetEntry url_drag_types [] =
@@ -87,8 +87,6 @@ static const GtkTargetEntry url_drag_types [] =
 	{ EPHY_DND_TEXT_TYPE,       0, 2 }
 };
 
-static void ephy_location_entry_class_init (EphyLocationEntryClass *klass);
-static void ephy_location_entry_init (EphyLocationEntry *le);
 static gboolean ephy_location_entry_reset_internal (EphyLocationEntry *, gboolean);
 
 static void textcell_data_func (GtkCellLayout *cell_layout,
@@ -102,6 +100,16 @@ static void extracell_data_func (GtkCellLayout *cell_layout,
 				 GtkTreeIter *iter,
 				 gpointer data);
 
+enum
+{
+	PROP_0,
+	PROP_LOCATION,
+	PROP_FAVICON,
+	PROP_LOCK_STOCK,
+	PROP_SHOW_LOCK,
+	PROP_SHOW_FAVICON
+};
+
 enum signalsEnum
 {
 	USER_CHANGED,
@@ -114,6 +122,59 @@ static gint signals[LAST_SIGNAL] = { 0 };
 
 G_DEFINE_TYPE (EphyLocationEntry, ephy_location_entry, GTK_TYPE_ENTRY)
 
+static void
+ephy_location_entry_set_property (GObject *object,
+				  guint prop_id,
+				  const GValue *value,
+				  GParamSpec *pspec)
+{
+	EphyLocationEntry *entry = EPHY_LOCATION_ENTRY (object);
+
+	switch (prop_id)
+	{
+	case PROP_LOCATION:
+		ephy_location_entry_set_location (entry,
+						  g_value_get_string (value));
+		break;
+	case PROP_FAVICON:
+		ephy_location_entry_set_favicon (entry,
+						 g_value_get_object (value));
+		break;
+	case PROP_LOCK_STOCK:
+		ephy_location_entry_set_lock_stock (entry,
+						    g_value_get_string (value));
+		break;
+	case PROP_SHOW_LOCK:
+		ephy_location_entry_set_show_lock (entry,
+						   g_value_get_boolean (value));
+		break;
+	case PROP_SHOW_FAVICON:
+		ephy_location_entry_set_show_favicon (entry,
+						      g_value_get_boolean (value));
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id,pspec);
+	}
+}
+
+static void
+ephy_location_entry_get_property (GObject *object,
+				  guint prop_id,
+				  GValue *value,
+				  GParamSpec *pspec)
+{
+	EphyLocationEntry *entry = EPHY_LOCATION_ENTRY (object);
+
+	switch (prop_id)
+	{
+	case PROP_LOCATION:
+		g_value_set_string (value, ephy_location_entry_get_location (entry));
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id,pspec);
+	}
+}
+
 static void
 ephy_location_entry_finalize (GObject *object)
 {
@@ -140,8 +201,70 @@ ephy_location_entry_class_init (EphyLocationEntryClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+	object_class->get_property = ephy_location_entry_get_property;
+	object_class->set_property = ephy_location_entry_set_property;
 	object_class->finalize = ephy_location_entry_finalize;
 
+	/**
+	* EphyLocationEntry:location:
+	*
+	* The current location.
+	*/
+	g_object_class_install_property (object_class,
+					 PROP_LOCATION,
+					 g_param_spec_string ("location",
+							      "Location",
+							      "The current location",
+							      "",
+							      G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+
+	/**
+	* EphyLocationEntry:favicon:
+	*
+	* The icon corresponding to the current location.
+	*/
+	g_object_class_install_property (object_class,
+					 PROP_FAVICON,
+					 g_param_spec_object ("favicon",
+							      "Favicon",
+							      "The icon corresponding to the current location",
+							      GDK_TYPE_PIXBUF,
+							      G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+
+	/**
+	* EphyLocationEntry:lock-stock-id:
+	*
+	* Stock id of the security icon.
+	*/
+	g_object_class_install_property (object_class,
+					 PROP_LOCK_STOCK,
+					 g_param_spec_string  ("lock-stock-id",
+							       "Lock Stock ID",
+							       "Stock id of the security icon",
+							       NULL,
+							       G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+
+	/**
+	* EphyLocationEntry:show-lock:
+	*
+	* If we should show the security icon.
+	*/
+	g_object_class_install_property (object_class,
+					 PROP_SHOW_LOCK,
+					 g_param_spec_boolean ("show-lock",
+							       "Show Lock",
+							       "If we should show the security icon",
+							       FALSE,
+							       G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+
+	g_object_class_install_property (object_class,
+					 PROP_SHOW_FAVICON,
+					 g_param_spec_boolean ("show-favicon",
+							       "Show Favicon",
+							       "Whether to show the favicon",
+							       TRUE,
+							       G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+
        /**
 	* EphyLocationEntry::user-changed:
 	* @entry: the object on which the signal is emitted
@@ -240,13 +363,13 @@ update_favicon (EphyLocationEntry *lentry)
 	/* Only show the favicon if the entry's text is the
 	 * address of the current page.
 	 */
-	if (priv->favicon != NULL && priv->original_address)
+	if (priv->show_favicon && priv->favicon != NULL && priv->original_address)
 	{
 		gtk_entry_set_icon_from_pixbuf (entry,
 						GTK_ENTRY_ICON_PRIMARY,
 						priv->favicon);
 	}
-	else
+	else if (priv->show_favicon)
 	{
 		/* Here we could consider using fallback favicon that matches
 		 * the page MIME type, though text/html should be good enough
@@ -256,6 +379,12 @@ update_favicon (EphyLocationEntry *lentry)
 						   GTK_ENTRY_ICON_PRIMARY,
 						   "text-html");
 	}
+	else
+	{
+		gtk_entry_set_icon_from_icon_name (entry,
+						   GTK_ENTRY_ICON_PRIMARY,
+						   NULL);
+	}
 }
 
 static void
@@ -780,6 +909,7 @@ ephy_location_entry_init (EphyLocationEntry *le)
 	p->block_update = FALSE;
 	p->saved_text = NULL;
 	p->show_lock = FALSE;
+	p->show_favicon = TRUE;
 	p->dns_prefetch_handler = 0;
 
 	ephy_location_entry_construct_contents (le);
@@ -807,9 +937,13 @@ free_prefetch_helper (PrefetchHelper *helper)
 static gboolean
 do_dns_prefetch (PrefetchHelper *helper)
 {
+#ifdef HAVE_WEBKIT2
+	/* TODO: Network Features */
+#else
 	SoupSession *session = webkit_get_default_session ();
 
 	soup_session_prepare_for_uri (session, helper->uri);
+#endif
 
 	helper->entry->priv->dns_prefetch_handler = 0;
 
@@ -908,9 +1042,8 @@ textcell_data_func (GtkCellLayout *cell_layout,
 					     &color);
 
 		att = pango_attr_foreground_new
-			(color.red, color.green, color.blue);
+		   (color.red * 65535, color.green * 65535, color.blue * 65535);
 		att->start_index = strlen (title)+1;
-
 		pango_attr_list_insert (list, att);
 	        g_free (title);
 	}
@@ -918,6 +1051,7 @@ textcell_data_func (GtkCellLayout *cell_layout,
 	{
 		ctext = title;
 	}
+	g_object_set (cell, "attributes", list, NULL);
 
 	g_value_init (&text, G_TYPE_STRING);
 	g_value_take_string (&text, ctext);
@@ -1033,15 +1167,66 @@ ephy_location_entry_set_completion (EphyLocationEntry *entry,
 	gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (completion),
 				       cell, "pixbuf", favicon_col);
 
+	/* Pixel-perfect aligment with the location entry favicon
+	 * (16x16). Consider that this /might/ depend on the theme.
+	 *
+	 * The GtkEntryCompletion can not be themed so we work-around
+	 * that with padding and fixed sizes.
+	 * For the first cell, this is:
+	 *
+	 * ___+++++iiiiiiiiiiiiiiii++__ttt...bbb++++++__
+	 *
+	 * _ = widget spacing, can not be handled (3 px)
+	 * + = padding (5 px) (ICON_PADDING_LEFT)
+	 * i = the icon (16 px) (ICON_CONTENT_WIDTH)
+	 * + = padding (2 px) (ICON_PADDING_RIGHT) (cut by the fixed_size)
+	 * _ = spacing between cells, can not be handled (2 px)
+	 * t = the text (expands)
+	 * b = bookmark icon (16 px)
+	 * + = padding (6 px) (BKMK_PADDING_RIGHT)
+	 * _ = widget spacing, can not be handled (2 px)
+	 *
+	 * Each character is a pixel.
+	 *
+	 * The text cell and the bookmark icon cell are much more
+	 * flexible in its aligment, because they do not have to align
+	 * with anything in the entry.
+	 */
+
+#define ROW_PADDING_VERT 2
+
+#define ICON_PADDING_LEFT 5
+#define ICON_CONTENT_WIDTH 16
+#define ICON_PADDING_RIGHT 2
+
+#define ICON_CONTENT_HEIGHT 16
+
+#define TEXT_PADDING_LEFT 0
+
+#define BKMK_PADDING_RIGHT 6
+
+	gtk_cell_renderer_set_padding
+		(cell, ICON_PADDING_LEFT, ROW_PADDING_VERT);
+	gtk_cell_renderer_set_fixed_size
+		(cell,
+		 (ICON_PADDING_LEFT + ICON_CONTENT_WIDTH + ICON_PADDING_RIGHT),
+		 ICON_CONTENT_HEIGHT);
+	gtk_cell_renderer_set_alignment (cell, 0.0, 0.5);
+
 	cell = gtk_cell_renderer_text_new ();
-	g_object_set (cell,
-		      "ellipsize", PANGO_ELLIPSIZE_END,
-		      "ellipsize-set", TRUE,
-		      NULL);
+	g_object_set (cell, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
 	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (completion),
 				    cell, TRUE);
 	gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (completion),
 				       cell, "text", text_col);
+
+	/* Pixel-perfect aligment with the text in the location entry.
+	 * See above.
+	 */
+	gtk_cell_renderer_set_padding
+		(cell, TEXT_PADDING_LEFT, ROW_PADDING_VERT);
+	gtk_cell_renderer_set_alignment (cell, 0.0, 0.5);
+
         /*
          * As the width of the entry completion is known in advance 
          * (as big as the entry you are completing on), we can set 
@@ -1059,6 +1244,7 @@ ephy_location_entry_set_completion (EphyLocationEntry *entry,
 					    NULL);
 
 	cell = gtk_cell_renderer_pixbuf_new ();
+	g_object_set (cell, "follow-state", TRUE, NULL);
 	gtk_cell_layout_pack_end (GTK_CELL_LAYOUT (completion),
 				  cell, FALSE);
 	gtk_cell_layout_set_cell_data_func (GTK_CELL_LAYOUT (completion),
@@ -1066,6 +1252,11 @@ ephy_location_entry_set_completion (EphyLocationEntry *entry,
 					    entry,
 					    NULL);
 
+	/* Pixel-perfect aligment. This just keeps the same margin from
+	 * the border than the favicon on the other side. See above. */
+	gtk_cell_renderer_set_padding
+		(cell, BKMK_PADDING_RIGHT, ROW_PADDING_VERT);
+
 	g_object_set (completion, "inline-selection", TRUE, NULL);
 	g_signal_connect (completion, "cursor-on-match",
 			  G_CALLBACK (cursor_on_match_cb), entry);
@@ -1114,7 +1305,7 @@ ephy_location_entry_set_location (EphyLocationEntry *entry,
 		}
 	}
 
-	if (address != NULL && strcmp (address, "about:blank") != 0)
+	if (address != NULL)
 	{
 		if (g_str_has_prefix (address, EPHY_ABOUT_SCHEME))
 			effective_text = g_strdup_printf ("about:%s",
@@ -1314,6 +1505,21 @@ ephy_location_entry_set_favicon (EphyLocationEntry *entry,
 	update_favicon (entry);
 }
 
+void
+ephy_location_entry_set_show_favicon (EphyLocationEntry *entry,
+				      gboolean show_favicon)
+{
+	EphyLocationEntryPrivate *priv;
+
+	g_return_if_fail (EPHY_IS_LOCATION_ENTRY (entry));
+
+	priv = entry->priv;
+
+	priv->show_favicon = show_favicon != FALSE;
+
+	update_favicon (entry);
+}
+
 /**
  * ephy_location_entry_set_show_lock:
  * @entry: an #EphyLocationEntry widget
