@@ -39,7 +39,11 @@
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 #include <libsoup/soup.h>
+#ifdef HAVE_WEBKIT2
+#include <webkit2/webkit2.h>
+#else
 #include <webkit/webkit.h>
+#endif
 #include <gnome-keyring.h>
 #include <gnome-keyring-memory.h>
 
@@ -165,6 +169,16 @@ typedef struct
 	guint num_checked;
 } PdmClearAllDialogButtons;
 
+#ifdef HAVE_WEBKIT2
+static WebKitCookieManager *
+get_cookie_manager ()
+{
+	WebKitWebContext *web_context;
+
+	web_context = webkit_web_context_get_default ();
+	return webkit_web_context_get_cookie_manager (web_context);
+}
+#else
 static SoupCookieJar*
 get_cookie_jar ()
 {
@@ -173,6 +187,7 @@ get_cookie_jar ()
 	session = webkit_get_default_session ();
 	return (SoupCookieJar*)soup_session_get_feature (session, SOUP_TYPE_COOKIE_JAR);
 }
+#endif
 
 static void
 clear_all_dialog_release_cb (PdmClearAllDialogButtons *data)
@@ -180,6 +195,7 @@ clear_all_dialog_release_cb (PdmClearAllDialogButtons *data)
 	g_slice_free (PdmClearAllDialogButtons, data);
 }
 
+#ifndef HAVE_WEBKIT2
 static void
 clear_all_cookies (SoupCookieJar *jar)
 {
@@ -191,6 +207,7 @@ clear_all_cookies (SoupCookieJar *jar)
 
 	soup_cookies_free (l);
 }
+#endif
 
 static void
 get_info_full_cb (GnomeKeyringResult result,
@@ -260,10 +277,17 @@ clear_all_dialog_response_cb (GtkDialog *dialog,
 		if (gtk_toggle_button_get_active
 			(GTK_TOGGLE_BUTTON (checkbuttons->checkbutton_cookies)))
 		{
+#ifdef HAVE_WEBKIT2
+			WebKitCookieManager *cookie_manager;
+
+			cookie_manager = get_cookie_manager ();
+			webkit_cookie_manager_delete_all_cookies (cookie_manager);
+#else
 			SoupCookieJar *jar;
 
 			jar = get_cookie_jar ();
 			clear_all_cookies (jar);
+#endif
 		}
 		if (gtk_toggle_button_get_active
 			(GTK_TOGGLE_BUTTON (checkbuttons->checkbutton_passwords)))
@@ -290,7 +314,11 @@ clear_all_dialog_response_cb (GtkDialog *dialog,
 
 			ephy_embed_single_clear_cache (single);
 
+#ifdef HAVE_WEBKIT2
+			/* TODO: Favicons */
+#else
 			webkit_favicon_database_clear (webkit_get_favicon_database ());
+#endif
 		}
 	}
 	gtk_widget_destroy (GTK_WIDGET (dialog));
@@ -519,7 +547,11 @@ pdm_cmd_delete_selection (PdmActionInfo *action)
 		path = gtk_tree_row_reference_get_path ((GtkTreeRowReference *)r->data);
 		gtk_tree_model_get_iter (model, &iter, path);
 		gtk_tree_model_get_value (model, &iter, action->data_col, &val);
+#ifdef HAVE_WEBKIT2
+		action->remove (action, g_value_get_string (&val));
+#else
 		action->remove (action, g_value_get_boxed (&val));
+#endif
 		g_value_unset (&val);
 
 		/* for cookies we delete from callback, for passwords right here */
@@ -625,7 +657,7 @@ show_cookies_properties (PdmDialog *dialog,
 
 	cookie_dialog = ephy_dialog_new_with_parent (parent);
 	ephy_dialog_construct (cookie_dialog,
-			       ephy_file ("epiphany.ui"),
+			       "/org/gnome/epiphany/epiphany.ui",
 			       "cookie_properties_dialog",
 			       NULL);
 
@@ -759,10 +791,13 @@ pdm_dialog_cookies_construct (PdmActionInfo *info)
 
 	info->model = GTK_TREE_MODEL (liststore);
 	g_object_unref (liststore);
-
+#ifdef HAVE_WEBKIT2
+	/* Cookies properties are not available in WebKit2 */
+#else
 	g_signal_connect (selection, "changed",
 			  G_CALLBACK(cookies_treeview_selection_changed_cb),
 			  dialog);
+#endif
 
 	renderer = gtk_cell_renderer_text_new ();
 
@@ -802,6 +837,21 @@ pdm_dialog_cookies_construct (PdmActionInfo *info)
 	setup_action (info);
 }
 
+#ifdef HAVE_WEBKIT2
+static void
+cookie_changed_cb (WebKitCookieManager *cookie_manager,
+		   PdmDialog *dialog)
+{
+	PdmActionInfo *info;
+
+	info = dialog->priv->cookies;
+
+	g_signal_handlers_disconnect_by_func (cookie_manager, cookie_changed_cb, dialog);
+	gtk_list_store_clear (GTK_LIST_STORE (info->model));
+	info->filled = FALSE;
+	info->fill (info);
+}
+#else
 static gboolean
 cookie_to_iter (GtkTreeModel *model,
 		const SoupCookie *cookie,
@@ -869,6 +919,7 @@ cookie_changed_cb (SoupCookieJar *jar,
 		info->add (info, (gpointer) soup_cookie_copy ((SoupCookie*)new_cookie));
 	}
 }
+#endif
 
 static gboolean
 cookie_host_to_iter (GtkTreeModel *model,
@@ -958,9 +1009,57 @@ compare_cookie_host_keys (GtkTreeModel *model,
 	return retval;
 }
 
+#ifdef HAVE_WEBKIT2
+static void
+get_domains_with_cookies_cb (WebKitCookieManager *cookie_manager,
+			     GAsyncResult *result,
+			     PdmActionInfo *info)
+{
+	gchar** domains;
+	guint i;
+
+	domains = webkit_cookie_manager_get_domains_with_cookies_finish (cookie_manager, result, NULL);
+	if (!domains)
+		return;
+
+	for (i = 0; domains[i]; i++)
+	{
+		info->add (info, domains[i]);
+	}
+
+	/* The array items have been consumed, so we need only to free the array. */
+	g_free (domains);
+
+	/* Now turn on sorting */
+	gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (info->model),
+					 COL_COOKIES_HOST_KEY,
+					 (GtkTreeIterCompareFunc) compare_cookie_host_keys,
+					 NULL, NULL);
+	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (info->model),
+					      COL_COOKIES_HOST_KEY,
+					      GTK_SORT_ASCENDING);
+
+	g_signal_connect (cookie_manager, "changed",
+			  G_CALLBACK (cookie_changed_cb), info->dialog);
+
+	info->filled = TRUE;
+	info->scroll_to (info);
+}
+#endif
+
 static void
 pdm_dialog_fill_cookies_list (PdmActionInfo *info)
 {
+#ifdef HAVE_WEBKIT2
+	WebKitCookieManager *cookie_manager;
+
+	g_assert (info->filled == FALSE);
+
+	cookie_manager = get_cookie_manager ();
+	webkit_cookie_manager_get_domains_with_cookies (cookie_manager, NULL,
+							(GAsyncReadyCallback) get_domains_with_cookies_cb,
+							info);
+#else
 	SoupCookieJar *jar;
 	GSList *list, *l;
 
@@ -992,6 +1091,7 @@ pdm_dialog_fill_cookies_list (PdmActionInfo *info)
 			  G_CALLBACK (cookie_changed_cb), info->dialog);
 
 	info->scroll_to (info);
+#endif
 }
 
 static void
@@ -1005,7 +1105,11 @@ static void
 pdm_dialog_cookie_add (PdmActionInfo *info,
 		       gpointer data)
 {
+#ifdef HAVE_WEBKIT2
+	char *domain = (char *) data;
+#else
 	SoupCookie *cookie = (SoupCookie *) data;
+#endif
 	GtkListStore *store;
 	GtkTreeIter iter;
 	int column[4] = { COL_COOKIES_HOST, COL_COOKIES_HOST_KEY, COL_COOKIES_NAME, COL_COOKIES_DATA };
@@ -1024,10 +1128,15 @@ pdm_dialog_cookie_add (PdmActionInfo *info,
 	g_value_init (&value[2], G_TYPE_STRING);
 	g_value_init (&value[3], SOUP_TYPE_COOKIE);
 
+#ifdef HAVE_WEBKIT2
+	g_value_set_static_string (&value[0], domain);
+	g_value_take_string (&value[1], ephy_string_collate_key_for_domain (domain, -1));
+#else
 	g_value_set_static_string (&value[0], cookie->domain);
 	g_value_take_string (&value[1], ephy_string_collate_key_for_domain (cookie->domain, -1));
 	g_value_set_static_string (&value[2], cookie->name);
 	g_value_take_boxed (&value[3], cookie);
+#endif
 
 	gtk_list_store_insert_with_valuesv (store, &iter, -1,
 					    column, value,
@@ -1043,9 +1152,17 @@ static void
 pdm_dialog_cookie_remove (PdmActionInfo *info,
 			  gpointer data)
 {
+#ifdef HAVE_WEBKIT2
+	const char *domain = (const char *) data;
+
+	webkit_cookie_manager_delete_cookies_for_domain (get_cookie_manager (),
+							 domain);
+
+#else
 	SoupCookie *cookie = (SoupCookie *) data;
 
 	soup_cookie_jar_delete_cookie (get_cookie_jar(), cookie);
+#endif
 }
 
 static void
@@ -1154,6 +1271,33 @@ passwords_show_toggled_cb (GtkWidget *button,
 	gtk_tree_view_column_set_visible (column, active);
 }
 
+static gboolean
+password_search_equal (GtkTreeModel *model,
+		       int column,
+		       const gchar *key,
+		       GtkTreeIter *iter,
+		       gpointer search_data)
+{
+	char *host, *user;
+	gboolean retval;
+
+	/* Note that this is function has to return FALSE for a *match* ! */
+
+	gtk_tree_model_get (model, iter, COL_PASSWORDS_HOST, &host, -1);
+	retval = strstr (host, key) == NULL;
+	g_free (host);
+	if (retval == FALSE)
+		return retval;
+
+	gtk_tree_model_get (model, iter, COL_PASSWORDS_USER, &user, -1);
+	retval = strstr (user, key) == NULL;
+	g_free (user);
+
+	return retval;
+}
+
+
+
 static void
 pdm_dialog_passwords_construct (PdmActionInfo *info)
 {
@@ -1236,6 +1380,11 @@ pdm_dialog_passwords_construct (PdmActionInfo *info)
 	gtk_tree_view_column_set_reorderable (column, TRUE);
 	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
 
+	/* Setup basic type ahead filtering */
+	gtk_tree_view_set_search_equal_func (treeview,
+					     (GtkTreeViewSearchEqualFunc) password_search_equal,
+					     dialog, NULL);
+
 	info->treeview = treeview;
 	setup_action (info);
 }
@@ -1466,7 +1615,7 @@ pdm_dialog_init (PdmDialog *dialog)
 	priv = dialog->priv = EPHY_PDM_DIALOG_GET_PRIVATE (dialog);
 
 	ephy_dialog_construct (EPHY_DIALOG (dialog),
-			       ephy_file ("epiphany.ui"),
+			       "/org/gnome/epiphany/epiphany.ui",
 			       "pdm_dialog",
 			       NULL);
 
@@ -1499,7 +1648,11 @@ pdm_dialog_init (PdmDialog *dialog)
 	cookies->scroll_to = pdm_dialog_cookie_scroll_to;
 	cookies->dialog = dialog;
 	cookies->remove_id = "cookies_remove_button";
+#ifdef HAVE_WEBKIT2
+	cookies->data_col = COL_COOKIES_HOST;
+#else
 	cookies->data_col = COL_COOKIES_DATA;
+#endif
 	cookies->scroll_to_host = NULL;
 	cookies->filled = FALSE;
 	cookies->delete_row_on_remove = FALSE;
@@ -1540,7 +1693,11 @@ pdm_dialog_finalize (GObject *object)
 	g_signal_handlers_disconnect_matched
 		(single, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, object);
 
+#ifdef HAVE_WEBKIT2
+	g_signal_handlers_disconnect_by_func (get_cookie_manager (), cookie_changed_cb, object);
+#else
 	g_signal_handlers_disconnect_by_func (get_cookie_jar (), cookie_changed_cb, object);
+#endif
 
 	dialog->priv->cookies->destruct (dialog->priv->cookies);
 	dialog->priv->passwords->destruct (dialog->priv->passwords);
